=== modified file 'bzrlib/smart/client.py'
--- bzrlib/smart/client.py	2008-04-23 01:12:44 +0000
+++ bzrlib/smart/client.py	2008-04-23 06:17:59 +0000
@@ -39,7 +39,7 @@
             request_encoder = protocol.ProtocolThreeRequester(request)
             response_handler = message.ConventionalResponseHandler()
             response_proto = protocol.ProtocolThreeDecoder(response_handler)
-            response_handler.setProtoAndMedium(response_proto, request)
+            response_handler.setProtoAndMediumRequest(response_proto, request)
         elif version == 2:
             request_encoder = protocol.SmartClientRequestProtocolTwo(request)
             response_handler = request_encoder

=== modified file 'bzrlib/smart/message.py'
--- bzrlib/smart/message.py	2008-04-23 05:36:35 +0000
+++ bzrlib/smart/message.py	2008-04-23 06:17:59 +0000
@@ -91,7 +91,46 @@
         pass
 
 
-class ConventionalResponseHandler(MessageHandler):
+class ResponseHandler(object):
+    """Abstract base class for an object that handles a smart response."""
+
+    def read_response_tuple(self, expect_body=False):
+        """Reads and returns the response tuple for the current request.
+        
+        :keyword expect_body: a boolean indicating if a body is expected in the
+            response.  Some protocol versions needs this information to know
+            when a response is finished.  If False, read_body_bytes should
+            *not* be called afterwards.  Defaults to False.
+        :returns: tuple of response arguments.
+        """
+        raise NotImplementedError(self.read_response_tuple)
+
+    def read_body_bytes(self, count=-1):
+        """Read and return some bytes from the body.
+
+        :param count: if specified, read up to this many bytes.  By default,
+            reads the entire body.
+        :returns: str of bytes from the response body.
+        """
+        raise NotImplementedError(self.read_body_bytes)
+
+    def read_streamed_body(self):
+        """Returns an iterable that reads and returns a series of body chunks.
+        """
+        raise NotImplementedError(self.read_streamed_body)
+
+    def cancel_read_body(self):
+        """Stop expecting a body for this response.
+
+        If expect_body was passed to read_response_tuple, this cancels that
+        expectation (and thus finishes reading the response, allowing a new
+        request to be issued).  This is useful if a response turns out to be an
+        error rather than a normal result with a body.
+        """
+        raise NotImplementedError(self.cancel_read_body)
+
+
+class ConventionalResponseHandler(MessageHandler, ResponseHandler):
 
     def __init__(self):
         MessageHandler.__init__(self)
@@ -104,9 +143,9 @@
         self._body_error_args = None
         self.finished_reading = False
 
-    def setProtoAndMedium(self, protocol_decoder, medium):
+    def setProtoAndMediumRequest(self, protocol_decoder, medium_request):
         self._protocol_decoder = protocol_decoder
-        self._medium = medium
+        self._medium_request = medium_request
 
     def byte_part_received(self, byte):
         if byte not in ['E', 'S']:
@@ -158,9 +197,9 @@
         if next_read_size == 0:
             # a complete request has been read.
             self.finished_reading = True
-            self._medium.finished_reading()
+            self._medium_request.finished_reading()
             return
-        bytes = self._medium.read_bytes(next_read_size)
+        bytes = self._medium_request.read_bytes(next_read_size)
         if bytes == '':
             # end of file encountered reading from server
             raise errors.ConnectionReset(

=== modified file 'bzrlib/smart/protocol.py'
--- bzrlib/smart/protocol.py	2008-04-23 01:12:44 +0000
+++ bzrlib/smart/protocol.py	2008-04-23 06:17:59 +0000
@@ -59,6 +59,37 @@
     return '\x01'.join(args) + '\n'
 
 
+class Requester(object):
+    """Abstract base class for an object that can issue requests on a smart
+    medium.
+    """
+
+    def call(self, *args):
+        """Make a remote call.
+
+        :param args: the arguments of this call.
+        """
+        raise NotImplementedError(self.call)
+
+    def call_with_body_bytes(self, args, body):
+        """Make a remote call with a body.
+
+        :param args: the arguments of this call.
+        :type body: str
+        :param body: the body to send with the request.
+        """
+        raise NotImplementedError(self.call_with_body_bytes)
+
+    def call_with_body_readv_array(self, args, body):
+        """Make a remote call with a readv array.
+
+        :param args: the arguments of this call.
+        :type body: iterable of (start, length) tuples.
+        :param body: the readv ranges to send with this request.
+        """
+        raise NotImplementedError(self.call_with_body_readv_array)
+
+
 class SmartProtocolBase(object):
     """Methods common to client and server"""
 
@@ -472,7 +503,8 @@
         return result
 
 
-class SmartClientRequestProtocolOne(SmartProtocolBase):
+class SmartClientRequestProtocolOne(SmartProtocolBase, Requester,
+        message.ResponseHandler):
     """The client-side protocol for smart version 1."""
 
     def __init__(self, request):
@@ -949,7 +981,7 @@
         self._write_end()
         
 
-class ProtocolThreeRequester(_ProtocolThreeEncoder):
+class ProtocolThreeRequester(_ProtocolThreeEncoder, Requester):
 
     def __init__(self, medium_request):
         _ProtocolThreeEncoder.__init__(self, medium_request.accept_bytes)

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2008-04-23 01:12:44 +0000
+++ bzrlib/tests/test_smart_transport.py	2008-04-23 06:17:59 +0000
@@ -1394,6 +1394,9 @@
     client_protocol_class = None
 
     def make_client_protocol_and_output(self, input_bytes=None):
+        """
+        :returns: a Request
+        """
         # This is very similar to
         # bzrlib.smart.client._SmartClient._build_client_protocol
         if input_bytes is None:
@@ -1412,7 +1415,8 @@
             requester = self.request_encoder(request)
             response_handler = message.ConventionalResponseHandler()
             response_protocol = self.response_decoder(response_handler)
-            response_handler.setProtoAndMedium(response_protocol, request)
+            response_handler.setProtoAndMediumRequest(
+                response_protocol, request)
             return requester, response_handler, output
 
     def make_client_protocol(self, input_bytes=None):
@@ -2387,7 +2391,8 @@
             StringIO(interrupted_body_stream), output)
         medium_request = client_medium.get_request()
         medium_request.finished_writing()
-        response_handler.setProtoAndMedium(protocol_decoder, medium_request)
+        response_handler.setProtoAndMediumRequest(
+            protocol_decoder, medium_request)
         stream = response_handler.read_streamed_body()
         self.assertEqual('chunk one', stream.next())
         self.assertEqual('chunk two', stream.next())
@@ -2433,7 +2438,7 @@
         """Make v3 response decoder using a conventional response handler."""
         response_handler = message.ConventionalResponseHandler()
         decoder = protocol.ProtocolThreeDecoder(response_handler)
-        response_handler.setProtoAndMedium(decoder, StubRequest())
+        response_handler.setProtoAndMediumRequest(decoder, StubRequest())
         return decoder, response_handler
 
     def test_trivial_response_decoding(self):

